
// ***********************************************************************
//
// TColorButton, lzh@43210.com, http://940801.cndev.net                                                            
//                                                                                   
// ***********************************************************************

unit ColorButtons;

interface

uses
  Windows, Messages, Classes, Controls, ExtCtrls, Buttons, Forms, Graphics,
  Dialogs, StdCtrls;

const
  MaxColorButtonNumber = 40;

type

  TGlyphType = (gtBackGround, gtText, gtLines);

  { TSubColorButton }

  TSubColorButton = class(TSpeedButton)
  protected
    procedure Paint; override;
  published
    property Color;
  end;

  TColorSelectedEvent = procedure(Sender: TObject; AColor: TColor) of object;

  { TColorButton }

  TColorButton = class(TCustomPanel)
  private
    FHoriMargin: integer;
    FTopMargin: integer;
    FBottomMargin: integer;
    FDragBarHeight: integer;
    FDragBarSpace: integer;
    FColorSpace: integer;
    FColorSize: integer;
    FColorSpaceTop: integer;
    FColorSpaceBottom: integer;
    FDropDownWidth: integer;
    FButtonHeight: integer;
    FDragBarRect: TRect;

    FColorDialog: TColorDialog;
    FGlyphType: TGlyphType;

    FMainButton: TSubColorButton;
    FSavedMainButtonWndProc: TWndMethod;

    FDropDownButton: TSpeedButton;
    FSavedDropDownButtonWndProc: TWndMethod;

    FDropDownForm: TForm;
    FSavedDropDownFormWndProc: TWndMethod;

    FColorButtons: array [0..MaxColorButtonNumber - 1] of TSubColorButton;
    FAutoButton: TSubColorButton;
    FOtherButton: TSpeedButton;

    FDropDownFlat: boolean;
    FFlat: boolean;
    FSelectedColor: TColor;

    FDropDownMove: boolean;
    FDropDownMoved: boolean;

    FOnColorSelected: TColorSelectedEvent;
    FOnUpdate: TNotifyEvent;
    FWordStyle: boolean;

    procedure CMFontChanged(var Msg: TMessage); message CM_FONTCHANGED;

    procedure MainButtonWndProc(var Msg: TMessage);
    procedure DropDownButtonWndProc(var Msg: TMessage);
    procedure DropDownFormWndProc(var Msg: TMessage);
    procedure ColorButtonClick(Sender: TObject);
    procedure ActionUpdate(Sender: TObject);
    
    procedure SetFormTitleBar(HasTitleBar: boolean);
    procedure AdjustDropDownForm(HasDragBar: boolean);
    procedure ShowDropDownForm;
    procedure SetDropDownWidth(const Value: integer);
    procedure SetDropDownFlat(const Value: boolean);
    procedure SetFlat(const Value: boolean);
    procedure SetCaptionAutoColor(const Value: string);
    function  GetCaptionAutoColor: string;
    procedure SetSelectedColor(const Value: TColor);
    procedure SetGlyphType(const Value: TGlyphType);
    function  GetCaptionOtherColor: string;
    procedure SetCaptionOtherColor(const Value: string);
    procedure SetDropDownMove(const Value: boolean);
    procedure SetWordStyle(const Value: boolean);
    procedure SetDropDownMeasure(const Index, Value: integer);
    function  GetCustomColors: TStrings;
    procedure SetCustomColors(const Value: TStrings);
    procedure SetDropDownCaption(const Value: string);
    function  GetDropDownCaption: string;
  protected
    procedure CreateWnd; override;
    procedure SetEnabled(Value: Boolean); override;
  public
    procedure AdjustSize; override;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property TopMargin: integer index 0 read FTopMargin write SetDropDownMeasure;
    property BottomMargin: integer index 1 read FBottomMargin write SetDropDownMeasure;
    property HoriMargin: integer index 2 read FHoriMargin write SetDropDownMeasure;
    property DragBarHeight: integer index 3 read FDragBarHeight write SetDropDownMeasure;
    property DragBarSpace: integer index 4 read FDragBarSpace write SetDropDownMeasure;
    property ColorSpace: integer index 5 read FColorSpace write SetDropDownMeasure;
    property ColorSpaceTop: integer index 6 read FColorSpaceTop write SetDropDownMeasure;
    property ColorSpaceBottom: integer index 7 read FColorSpaceBottom write SetDropDownMeasure;
    property ColorSize: integer index 8 read FColorSize write SetDropDownMeasure;
    property ButtonHeight: integer index 9 read FButtonHeight write SetDropDownMeasure;

    property CaptionAutoColor: string read GetCaptionAutoColor write SetCaptionAutoColor;
    property CaptionOtherColor: string read GetCaptionOtherColor write SetCaptionOtherColor;
    property DropDownCaption: string read GetDropDownCaption write SetDropDownCaption;

    property DropDownWidth: integer read FDropDownWidth write SetDropDownWidth;

    property DropDownMove: boolean read FDropDownMove write SetDropDownMove;
    property DropDownFlat: boolean read FDropDownFlat write SetDropDownFlat default true;
    property Flat: boolean read FFlat write SetFlat;
    property SelectedColor: TColor read FSelectedColor write SetSelectedColor;
    property GlyphType: TGlyphType read FGlyphType write SetGlyphType;
    property WordStyle: boolean read FWordStyle write SetWordStyle;
    property CustomColors: TStrings read GetCustomColors write SetCustomColors;

    property OnColorSelected: TColorSelectedEvent read FOnColorSelected write FOnColorSelected;
    property OnUpdate: TNotifyEvent read FOnUpdate write FOnUpdate;

    property Align;
//*    property Alignment;
    property Anchors;
//*    property AutoSize;
//*    property BevelInner;
//*    property BevelOuter;
//*    property BevelWidth;
    property BiDiMode;
//*    property BorderWidth;
//*    property BorderStyle;
//*    property Caption;
//*    property Color;
    property Constraints;
//*    property Ctl3D;
//    property UseDockManager default True;
//    property DockSite;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
//*    property FullRepaint;
    property Font;
//*    property Locked;
    property ParentBiDiMode;
//*    property ParentColor;
//*    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
//    property OnDockDrop;
//    property OnDockOver;
//*    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
//    property OnStartDock;
    property OnStartDrag;
//    property OnUnDock;
  end;

procedure Register;

implementation

{$R *.res}

uses
  ActnList, Math;

const
  MinButtonHeight  = 22;
  MinButtonWidth   = 23;
  MinDropDownWidth = 9;
  MinColorSize     = 18;
  MinColorSpace    = 0;
  MinColorSpaceTop = 4;
  MinColorSpaceBottom = 4;
  MinTopMargin     = 2;
  MinBottomMargin  = 4;
  MinHoriMargin    = 7;
  MinDragBarHeight = 7;
  MinDragBarSpace  = 3;

  LineColorButtonCount = 8;

  SubColorButtonColors: array [0..MaxColorButtonNumber - 1] of TColor =(
      $000000, $003399, $003333, $003300, $663300, $800000, $993333, $333333,
      $000080, $0066FF, $008080, $008000, $808000, $FF0000, $996666, $808080,
      $0000FF, $0099FF, $00CC99, $669933, $CCCC33, $FF6633, $800080, $999999,
      $FF00FF, $00CCFF, $00FFFF, $00FF00, $FFFF00, $FFCC00, $663399, $C0C0C0,
      $CC99FF, $99CCFF, $99FFFF, $CCFFCC, $FFFFCC, $FFCC99, $FF99CC, $FFFFFF);

procedure Register;
begin
  RegisterComponents('940801', [TColorButton]);
end;

{ TSubColorButton }

procedure TSubColorButton.Paint;
var
  C, S, X, Y: integer;
  R: TRect;
begin
  inherited Paint;

  R:= Rect(0, 0, Width, Height);

  if Flat then
  begin
    R.Right:= R.Right - 1;
    R.Bottom:= R.Bottom - 1;
  end;
  
  if not Flat and (FState = bsDown) then
  begin
    R.Left:= R.Left + 1;
    R.Top:= R.Top + 1;
  end;

  if not Flat and (FState = bsUp) then
  begin
    R.Right:= R.Right - 1;
    R.Bottom:= R.Bottom - 1;
  end;

  with Canvas do
    if Glyph.Handle <> 0 then
    begin
      X:= ((Width + 1) div 2) - 8 + Integer(FState in [bsDown]);
      Y:= ((Height + 1) div 2) + 4 + Integer(FState in [bsDown]);
      if Enabled then
      begin
        Pen.Color:= Color;
        Brush.Color:= Color;
      end
      else
      begin
        Pen.Color:= clInactiveCaption;
        Brush.Color:= clInactiveCaption;
      end;
      Rectangle(X, Y, X + 16, Y + 4);
    end
    else if Caption = '' then
    begin
      C:= (R.Bottom - R.Top) div 6 + 1;
      if Enabled then
      begin
        Pen.Color:= clGray;
        Brush.Color:= Color;
      end
      else
      begin
        Pen.Color:= clInactiveCaption;
        Brush.Color:= clBtnFace;
      end;
      Brush.Style:= bsSolid;
      Rectangle(R.Left + C, R.Top + C, R.Right - C + 1, R.Bottom - C + 1);
    end
    else
    begin
      C:= (R.Bottom - R.Top) div 6 + 3;
      S:= (R.Bottom - R.Top) div 7;
      if Enabled then
        Pen.Color:= clGray
      else
        Pen.Color:= clInactiveCaption;
      Brush.Style:= bsClear;
      Polygon([Point(R.Left + S, R.Top + S), Point(R.Right - S, R.Top + S), Point(R.Right - S, R.Bottom - S), Point(R.Left + S, R.Bottom - S)]);
      if Enabled then
      begin
        Pen.Color:= clGray;
        Brush.Color:= Color;
      end
      else
      begin
        Pen.Color:= clInactiveCaption;
        Brush.Color:= clBtnFace;
      end;
      Brush.Style:= bsSolid;

      Rectangle(R.Left + C + 1, R.Top + C, R.Bottom - C + 2 + 1, R.Bottom - C + 2);
    end;
end;

{ TColorButton }

procedure TColorButton.ActionUpdate(Sender: TObject);
begin
  if Assigned(FOnUpdate) then FOnUpdate(Self);
end;

procedure TColorButton.AdjustDropDownForm(HasDragBar: boolean);
var
  I: integer;
  ButtonLine: integer;
  TempHeight: integer;
begin
  FDropDownForm.DisableAlign;
  ButtonLine:= (MaxColorButtonNumber + LineColorButtonCount - 1) div LineColorButtonCount;

  if not HasDragBar then
    TempHeight:= - FDragBarHeight
  else
    TempHeight:= 0;

  with FDropDownForm do
  begin
    ClientWidth:= FHoriMargin * 2 + FColorSize * LineColorButtonCount + FColorSpace * (LineColorButtonCount - 1);
    ClientHeight:= FTopMargin + FBottomMargin + FDragBarHeight + FDragBarSpace + FColorSpaceTop + FColorSpaceBottom + FColorSize * ButtonLine + FColorSpace * (ButtonLine - 1) + FButtonHeight * 2 + TempHeight;
  end;

  if HasDragBar then
  begin
    FDragBarRect.Left:= (FHoriMargin + 1) div 2;
    FDragBarRect.Top:= FTopMargin;
    FDragBarRect.Right:= FDropDownForm.ClientWidth - (FHoriMargin + 1) div 2;
    FDragBarRect.Bottom:= FDragBarRect.Top + FDragBarHeight;
  end
  else
    FDragBarRect:= Rect(0, 0, 0, 0);  

  FAutoButton.SetBounds(FHoriMargin, FTopMargin + FDragBarHeight + FDragBarSpace + TempHeight, FDropDownForm.ClientWidth - FHoriMargin * 2, FButtonHeight);

  for I:= 0 to MaxColorButtonNumber - 1 do
    FColorButtons[I].SetBounds(FAutoButton.Left + (I mod LineColorButtonCount) * (FColorSpace + FColorSize),
                               FAutoButton.Top + FAutoButton.Height + FColorSpaceTop + (I div LineColorButtonCount) * (FColorSpace + FColorSize),
                               FColorSize,
                               FColorSize);

  FOtherButton.SetBounds(FAutoButton.Left,
                         FColorButtons[MaxColorButtonNumber - 1].Top + FColorSize + FColorSpaceBottom,
                         FAutoButton.Width,
                         FButtonHeight);
end;

procedure TColorButton.AdjustSize;
begin
  if FDropDownWidth < MinDropDownWidth then
    FDropDownWidth:= MinDropDownWidth;
  if (Width - FDropDownWidth) < MinButtonWidth then
    Width:= MinButtonWidth + FDropDownWidth;
  if Height < MinButtonHeight then
    Height:= MinButtonHeight;

  FMainButton.SetBounds(0, 0, Width - FDropDownWidth, Height);

  FDropDownButton.SetBounds(FMainButton.Width, 0, FDropDownWidth, Height);

  inherited AdjustSize;
end;

procedure TColorButton.CMFontChanged(var Msg: TMessage);
begin
  inherited;
  FDropDownForm.Font.Assign(Font);
end;

procedure TColorButton.ColorButtonClick(Sender: TObject);
begin
  if not FDropDownMoved then FDropDownForm.Hide;
  if TSpeedButton(Sender).Tag = FOtherButton.Tag then
  begin
    FColorDialog.Color:= FSelectedColor;
    if FColorDialog.Execute then
      SetSelectedColor(FColorDialog.Color)
    else
      Exit;
  end
  else
  begin
    TSubColorButton(Sender).Down:= true;
    SetSelectedColor(TSubColorButton(Sender).Color);
  end;
end;

constructor TColorButton.Create(AOwner: TComponent);
var
  I: integer;
begin
  inherited Create(AOwner);

  ControlStyle:= ControlStyle - [csAcceptsControls, csSetCaption] + [csOpaque];
  BevelOuter:= bvNone;
  Locked:= true;           
  Width:= MinButtonWidth + MinDropDownWidth;
  Height:= MinButtonHeight;

  FDropDownWidth:= MinDropDownWidth;

  FSelectedColor:= clDefault;
  FGlyphType:= gtText;

  FMainButton:= TSubColorButton.Create(Self);
  with FMainButton do
  begin
    Parent:= Self;
    NumGlyphs:= 2;
    Color:= clDefault;
    Glyph.Handle:= LoadBitmap(hInstance, 'TCOLORBUTTON_TEXT');

    FSavedMainButtonWndProc:= WindowProc;
    WindowProc:= MainButtonWndProc;
  end;

  FDropDownButton:= TSpeedButton.Create(Self);
  with FDropDownButton do
  begin
    Parent:= Self;
    GroupIndex:= 2;
    AllowAllUp:= true;
    Glyph.Handle:= LoadBitmap(HInstance, 'TCOLORBUTTON_DROPDOWN');

    FSavedDropDownButtonWndProc:= WindowProc;
    WindowProc:= DropDownButtonWndProc;
  end;

  FDropDownForm:= TForm.Create(Self);
  with FDropDownForm do
  begin
    AutoScroll:= false;
    BorderStyle:= bsToolWindow;
    FormStyle:= fsStayOnTop;
    Caption:= 'Text Color Window';

    FSavedDropDownFormWndProc:= WindowProc;
    WindowProc:= DropDownFormWndProc;
  end;

  FAutoButton:= TSubColorButton.Create(FDropDownForm);

  with FAutoButton do
  begin
    Parent:= FDropDownForm;
    GroupIndex:= 1;
    Tag:= MaxColorButtonNumber + 1;
    Down:= true;
    AllowAllUp:= true;
    Color:= clDefault;
    Caption:= 'Automatic';

    OnClick:= ColorButtonClick;
  end;

  for I:= 0 to MaxColorButtonNumber - 1 do
  begin
    FColorButtons[I]:= TSubColorButton.Create(FDropDownForm);
    with FColorButtons[I] do
    begin
      Parent:= FDropDownForm;
      GroupIndex:= 1;
      AllowAllUp:= true;
      Color:= SubColorButtonColors[I];
      Tag:= I;
      OnClick:= ColorButtonClick;
    end;
  end;

  FOtherButton:= TSpeedButton.Create(FDropDownForm);
  with FOtherButton do
  begin
    Parent:= FDropDownForm;
    Tag:= MaxColorButtonNumber + 2;
    Caption:= 'Other Colors...';

    OnClick:= ColorButtonClick;
  end;

  SetWordStyle(true);

  FColorDialog:= TColorDialog.Create(Self);
  FColorDialog.Options:= [cdFullOpen, cdSolidColor, cdAnyColor];

  Action:= TAction.Create(Self);
  Action.OnUpdate:= ActionUpdate;
end;

procedure TColorButton.CreateWnd;
begin
  inherited;
  AdjustSize;
end;

destructor TColorButton.Destroy;
begin
  //we do not need free sub components when destroy, because we set the sub components' parent as TColorButton.

  if FDropDownForm.Visible then FDropDownForm.Hide; //because we replaced message handler, we have to hide the form at here.
  Action.Free;
  inherited Destroy;
end;

procedure TColorButton.DropDownButtonWndProc(var Msg: TMessage);
begin
  FSavedDropDownButtonWndProc(Msg);
  case Msg.Msg of
    CM_MOUSEENTER:
      if Msg.WParam <> - 1 then
        FMainButton.Perform(Msg.Msg, -1, -1);
    CM_MOUSELEAVE:
      if not FDropDownButton.Down then
      begin
        if Msg.WParam <> -1 then FMainButton.Perform(Msg.Msg, -1, -1)
      end
      else
      begin
        FDropDownButton.Perform(CM_MOUSEENTER, -1, -1);
      end;
    WM_LBUTTONDOWN:
      ShowDropDownForm;
  end;
end;

procedure TColorButton.DropDownFormWndProc(var Msg: TMessage);
var
  pt: TPoint;
begin
  if not FDropDownMoved then
    case Msg.Msg of
      CM_MOUSELEAVE:
        InvaliDateRect(FDropDownForm.Handle, @FDragBarRect, false);
      WM_MOUSEMOVE:
        if not PtInRect(Rect(FDragBarRect.Left - 2, FDragBarRect.Top - 2, FDragBarRect.Right + 2, FDragBarRect.Bottom + 2), SmallPointToPoint(TWMMouseMove(Msg).Pos)) then Exit;
    end;

  FSavedDropDownFormWndProc(Msg);
  
  case Msg.Msg of
    WM_Paint:
    begin
      if FDropDownMoved then Exit;
      with FDropDownForm.Canvas do
      begin
        GetCursorPos(pt);
        pt:= FDropDownForm.ScreenToClient(pt);
        if PtInRect(FDragBarRect, pt) then
          Brush.Color:= clActiveCaption
        else
          Brush.Color:= clInactiveCaption;
        Brush.Style:= bsSolid;
        FillRect(FDragBarRect);
      end;
    end;
    WM_KILLFOCUS:
      if not FDropDownMoved then FDropDownForm.Hide;
    CM_SHOWINGCHANGED:
      if not FDropDownForm.Visible then
      begin
        FDropDownButton.Down:= false;
        FDropDownButton.Perform(CM_MOUSELEAVE, -1, -1);
        FMainButton.Perform(CM_MOUSELEAVE, -1, -1);
      end;
    WM_NCHITTEST:
      if not FDropDownMoved and (Msg.Result = HTClient) and PtInRect(FDragBarRect, FDropDownForm.ScreenToClient(SmallPointToPoint(TWMNCHitTest(Msg).Pos))) then
      begin
        Msg.Result:= HTCaption;
        InvaliDateRect(FDropDownForm.Handle, @FDragBarRect, false);
      end;
    WM_SYSCOMMAND:
      if not FDropDownMoved and (TWMSysCommand(Msg).CmdType and SC_MOVE > 0) then
      begin
        SetFormTitleBar(true);
        FDropDownForm.Invalidate;
        AdjustDropDownForm(false);
        FDropDownButton.Down:= false;
        FDropDownButton.Perform(CM_MOUSELEAVE, 0, 0);
      end;
  end;
end;

function TColorButton.GetCaptionAutoColor: string;
begin
  Result:= FAutoButton.Caption;
end;

function TColorButton.GetCaptionOtherColor: string;
begin
  Result:= FOtherButton.Caption;
end;

function TColorButton.GetCustomColors: TStrings;
begin
  Result:= FColorDialog.CustomColors;
end;

function TColorButton.GetDropDownCaption: string;
begin
  Result:= FDropDownForm.Caption;
end;

procedure TColorButton.MainButtonWndProc(var Msg: TMessage);
begin
  FSavedMainButtonWndProc(Msg);
  case Msg.Msg of
    CM_MOUSEENTER:
      if Msg.WParam <> - 1 then
        FDropDownButton.Perform(Msg.Msg, -1, -1);
    CM_MOUSELEAVE:
    begin
      if FDropDownForm.Visible and not FDropDownMoved and (Msg.WParam <> -1) then
        FMainButton.Perform(CM_MOUSEENTER, -1, -1);
      if Msg.WParam <> - 1 then
        FDropDownButton.Perform(Msg.Msg, -1, -1);
    end
  end;
end;

procedure TColorButton.SetCaptionAutoColor(const Value: string);
begin
  FAutoButton.Caption:= Value;
end;

procedure TColorButton.SetCaptionOtherColor(const Value: string);
begin
  FOtherButton.Caption:= Value;
end;

procedure TColorButton.SetCustomColors(const Value: TStrings);
begin
  FColorDialog.CustomColors.Assign(Value);
end;

procedure TColorButton.SetDropDownCaption(const Value: string);
begin
  FDropDownForm.Caption:= Value;
end;

procedure TColorButton.SetDropDownFlat(const Value: boolean);
var
  I: integer;
begin
  if FDropDownFlat <> Value then
  begin
    FDropDownFlat:= Value;

    FAutoButton.Flat:= Value;
    for I:= 0 to MaxColorButtonNumber - 1 do
      FColorButtons[I].Flat:= Value;
    FOtherButton.Flat:= Value;
  end;  
end;

procedure TColorButton.SetDropDownMeasure(const Index, Value: integer);
var
  MeasureItem: PInteger;
  MeasureConst: integer;
begin
  case Index of
    0:
    begin
      MeasureItem:= @FTopMargin;
      MeasureConst:= MinTopMargin;
    end;
    1:
    begin
      MeasureItem:= @FBottomMargin;
      MeasureConst:= MinBottomMargin;
    end;
    2:
    begin
      MeasureItem:= @FHoriMargin;
      MeasureConst:= MinHoriMargin;
    end;
    3:
    begin
      MeasureItem:= @FDragBarHeight;
      MeasureConst:= MinDragBarHeight;
    end;
    4:
    begin
      MeasureItem:= @FDragBarSpace;
      MeasureConst:= MinDragBarSpace;
    end;
    5:
    begin
      MeasureItem:= @FColorSpace;
      MeasureConst:= MinColorSpace;
    end;
    6:
    begin
      MeasureItem:= @FColorSpaceTop;
      MeasureConst:= MinColorSpaceTop;
    end;
    7:
    begin
      MeasureItem:= @FColorSpaceBottom;
      MeasureConst:= MinColorSpaceBottom;
    end;
    8:
    begin
      MeasureItem:= @FColorSize;
      MeasureConst:= MinColorSize;
    end;
    9:
    begin
      MeasureItem:= @FButtonHeight;
      MeasureConst:= MinButtonHeight;
    end;
    else
      Exit;
  end;
  if MeasureItem^ = Value then Exit;

  MeasureItem^:= Value;
  FWordStyle:= false;
  if MeasureItem^ < MeasureConst then MeasureItem^:= MeasureConst;
  if FDropDownForm.Visible then AdjustDropDownForm(not FDropDownMoved);
end;

procedure TColorButton.SetDropDownMove(const Value: boolean);
begin
  FDropDownMove:= Value;
  if FDropDownMoved then FDropDownForm.Close;
end;

procedure TColorButton.SetDropDownWidth(const Value: integer);
begin
  FDropDownWidth:= Value;
  AdjustSize;
end;

procedure TColorButton.SetEnabled(Value: Boolean);
var
  I: integer;
begin
  inherited SetEnabled(Value);
  FMainButton.Enabled:= Value;
  FDropDownButton.Enabled:= Value;
  FAutoButton.Enabled:= Value;
  for I:= 0 to MaxColorButtonNumber - 1 do
    FColorButtons[I].Enabled:= Value;
  FOtherButton.Enabled:= Value;
end;

procedure TColorButton.SetFlat(const Value: boolean);
begin
  if FFlat <> Value then
  begin
    FFlat:= Value;
    FMainButton.Flat:= Value;
    FDropDownButton.Flat:= Value;
  end;
end;

procedure TColorButton.SetFormTitleBar(HasTitleBar: boolean);
var
  WinStyle: LongInt;
begin
  WinStyle:= GetWindowLong(FDropDownForm.Handle, GWL_STYLE);
  if HasTitleBar then
    WinStyle:= WinStyle or WS_CAPTION
  else
    WinStyle:= WinStyle and not WS_CAPTION or WS_DLGFRAME;
  SetWindowLong(FDropDownForm.Handle, GWL_STYLE, WinStyle);
  if HasTitleBar then
    FDropDownForm.Height:= FDropDownForm.Height + GetSystemMetrics(SM_CYCAPTION)
  else
    FDropDownForm.Height:= FDropDownForm.Height - GetSystemMetrics(SM_CYCAPTION);
  FDropDownMoved:= HasTitleBar;
end;

procedure TColorButton.SetGlyphType(const Value: TGlyphType);
begin
  FGlyphType:= Value;
  case Value of
    gtBackGround:
      FMainButton.Glyph.Handle:= LoadBitmap(hInstance, 'TCOLOTBUTTON_BACKGROUD');
    gtLines:
      FMainButton.Glyph.Handle:= LoadBitmap(hInstance, 'TCOLORBUTTON_LINES');
    gtText:
      FMainButton.Glyph.Handle:= LoadBitmap(hInstance, 'TCOLORBUTTON_TEXT');
  end;
end;

procedure TColorButton.SetSelectedColor(const Value: TColor);
var
  I: integer;
begin
  if FSelectedColor <> Value then
  begin
    FMainButton.Color:= Value;
    FSelectedColor:= Value;

    if FAutoButton.Color = Value then
      FAutoButton.Down:= true
    else
    begin
      FAutoButton.Down:= false;
      for I:= 0 to MaxColorButtonNumber - 1 do
        if FColorButtons[I].Color = Value then
        begin
          FColorButtons[I].Down:= true;
          Break;
        end
        else
          FColorButtons[I].Down:= false;
    end;

    if Assigned(FOnColorSelected) then FOnColorSelected(Self, FSelectedColor);
  end;
end;

procedure TColorButton.SetWordStyle(const Value: boolean);
begin
  if FWordStyle <> Value then
  begin
    FWordStyle:= Value;
    if FWordStyle then
    begin
      FWordStyle:= Value;
      
      SetFlat(true);
      SetDropDownFlat(true);

      FButtonHeight:= MinButtonHeight;
      FDropDownWidth:= MinDropDownWidth;
      FColorSize:= MinColorSize;
      FColorSpace:= MinColorSpace;
      FColorSpaceTop:= MinColorSpaceTop;
      FColorSpaceBottom:= MinColorSpaceBottom;
      FTopMargin:= MinTopMargin;
      FBottomMargin:= MinBottomMargin;
      FHoriMargin:= MinHoriMargin;
      FDragBarHeight:= MinDragBarHeight;
      FDragBarSpace:= MinDragBarSpace;

      FDropDownMove:= true;
      AdjustDropDownForm(not FDropDownMoved);
    end;
  end;
end;

procedure TColorButton.ShowDropDownForm;
var
  pt: TPoint;
begin
  SetFormTitleBar(false);
  AdjustDropDownForm(FDropDownMove);

  pt:= ClientToScreen(Point(FMainButton.Left, FMainButton.Top));

  FDropDownForm.Left:= pt.X;
  if (FDropDownForm.Left + FDropDownForm.Width) > Screen.Width then
    FDropDownForm.Left:= Screen.Width - FDropDownForm.Width;
  FDropDownForm.Top:= pt.Y + Height;
  if (FDropDownForm.Top + FDropDownForm.Height) > Screen.Height then
    FDropDownForm.Top:= pt.Y - FDropDownForm.Height;

  FDropDownForm.Show;
end;

end.
